@@ -80,7 +80,7 @@ compute.2d <- function(y, f, s, family=c("clossR", "closs", "gloss", "qloss")){
         sqrt(2/pi)*u/s^3*exp(-u^2/(2*s^2))
 }
 
-nclreg_fit <- function(x,y, weights, offset=NULL, cost=0.5, rfamily=c("clossR", "closs", "gloss", "qloss"), s=NULL, fk=NULL, iter=10, del=1e-10, nlambda=100, lambda=NULL, lambda.min.ratio=ifelse(nobs<nvars,.05, .001),alpha=1, gamma=3, standardize=TRUE, penalty.factor = NULL, maxit=1000, type.init="bst", mstop.init=10, nu.init=0.1, direction=c("bwd", "fwd"), eps=.Machine$double.eps, trace=FALSE, penalty=c("enet","mnet","snet"), type.path=c("active", "naive", "onestep")){ 
+nclreg_fit <- function(x,y, weights, offset=NULL, cost=0.5, rfamily=c("clossR", "closs", "gloss", "qloss"), s=NULL, fk=NULL, iter=10, del=1e-10, penalty=c("enet","mnet","snet"), nlambda=100, lambda=NULL, type.path=c("active", "nonactive", "onestep"), decreasing=FALSE, lambda.min.ratio=ifelse(nobs<nvars,.05, .001),alpha=1, gamma=3, standardize=TRUE, penalty.factor = NULL, maxit=1000, type.init="bst", mstop.init=10, nu.init=0.1, eps=.Machine$double.eps, thresh=1e-6, trace=FALSE){
 ### compute h value
     compute.h <- function(rfamily, y, fk_old, s, B){
         if(rfamily=="clossR")
@@ -92,18 +92,26 @@ nclreg_fit <- function(x,y, weights, offset=NULL, cost=0.5, rfamily=c("clossR",
     call <- match.call()
     rfamily <- match.arg(rfamily)
     penalty <- match.arg(penalty)
-    direction <- match.arg(direction)
     type.path <- match.arg(type.path)
-    if(type.path=="active") direction <- "bwd"
-    if(!is.null(lambda) && type.path == "active"){
-        if (length(lambda) > 1 && any(diff(lambda) < 0))
-	    stop("for type.path='active', the provided lambda sequence must be increasing\n")
+    if(type.path=="active")
+	    active <- 1
+    else active <- 0
+    if (!is.null(lambda) && length(lambda) > 1 && all(lambda == cummin(lambda))){
+	    decreasing <- TRUE
+	    if(type.path=="active")
+	      warnings("choose type.path='nonactive' with increasing sequence of lambda, or let lambda=rev(lambda) as computed below\n")
+	    lambda <- rev(lambda)
     }
+    else if(!is.null(lambda) && length(lambda) > 1 && all(lambda == cummax(lambda)))
+	    decreasing <- FALSE
+    else if(is.null(lambda) && decreasing && type.path=="active")
+	    stop("set decreasing=FALSE or type.path='nonactive'")
     if(rfamily %in% c("closs", "gloss", "qloss"))
         if(!all(names(table(y)) %in% c(1, -1)))
             stop("response variable must be 1/-1 for family ", rfamily, "\n")
     nm <- dim(x)
     nobs <- n <- nm[1]
+    if(length(y)!=n) stop("length of y is different from row of x\n")
     nvars <- m <- nm[2]
     B <- bfunc(family=rfamily, s=s)
     if(missing(weights)) weights=rep(1,nobs)
@@ -115,6 +123,8 @@ nclreg_fit <- function(x,y, weights, offset=NULL, cost=0.5, rfamily=c("clossR",
     if( !is.null(weights) && any(weights < 0) ){
         stop("negative weights not allowed")
     }
+    if (is.null(offset))
+	                        offset <- rep.int(0, nobs)
     pentype <- switch(penalty,
                       "enet"=1,
                       "mnet"=2,
@@ -208,7 +218,7 @@ nclreg_fit <- function(x,y, weights, offset=NULL, cost=0.5, rfamily=c("clossR",
         lambda <- glmreg_fit(x=x*sqrt(B), y=h*sqrt(B), weights=weights, offset=offset, lambda.min.ratio=lambda.min.ratio, nlambda=nlambda, alpha=alpha,gamma=gamma, rescale=FALSE, standardize=FALSE, penalty.factor = penalty.factor, maxit=1, eps=eps, family="gaussian", penalty=penalty)$lambda
 ### with type.init, add two different lambda sequences and make lambda values flexible to have different solution paths
         if(type.init %in% c("bst", "heu")){
-            if(direction=="bwd")#{### solution path backward direction
+            if(!decreasing)#{### solution path backward direction
                 lambda <- rev(lambda)
         }
                                         #cat("lambda in method A", lambda[1], "\n")
@@ -250,6 +260,17 @@ nclreg_fit <- function(x,y, weights, offset=NULL, cost=0.5, rfamily=c("clossR",
         nlambda <- length(lambda)
     beta <- matrix(0, ncol=nlambda, nrow=m)
     b0 <- rep(0, nlambda)
+    ### initialize start values -begin
+    fk_old <- RET$fitted.values
+    h <- compute.h(rfamily, y, fk_old, s, B)
+    if(any(is.nan(h))){
+      cat('h has NaN, fk_old used to compute h at beginning', fk_old)
+      h <- y
+    }
+    tmp <- init(w, h, offset, family="gaussian")
+    mustart <- tmp$mu
+    etastart <- tmp$eta
+    ### initialize the start values -end
     stopit <- FALSE
 ### for each element of the lambda sequence, iterate until convergency
     typeA <- function(beta, b0){
@@ -265,7 +286,7 @@ nclreg_fit <- function(x,y, weights, offset=NULL, cost=0.5, rfamily=c("clossR",
             while(d1 > del && k <= iter){
                 fk_old <- RET$fitted.values
                 h <- compute.h(rfamily, y, fk_old, s, B)
-                if(any(is.nan(h))){ # exit loop 
+                if(any(is.nan(h))){ # exit loop
                     stopit <- TRUE
                     break
                 }
@@ -306,7 +327,7 @@ nclreg_fit <- function(x,y, weights, offset=NULL, cost=0.5, rfamily=c("clossR",
     }
 ### only for direction="bwd", cycle through only on active set. for each element of the lambda sequence, iterate until convergency
     typeB <- function(beta, b0){
-        i <- 1
+	i <- 1
 	x.act <- x ### current active set
 	start.act <- start
 	m.act <- dim(x.act)[2]
@@ -322,29 +343,71 @@ nclreg_fit <- function(x,y, weights, offset=NULL, cost=0.5, rfamily=c("clossR",
             while(d1 > del && k <= iter){
                 fk_old <- RET$fitted.values
                 h <- compute.h(rfamily, y, fk_old, s, B)
-                if(any(is.nan(h))){ # exit loop 
-                    stopit <- TRUE
+	    	if(any(is.nan(h))){ # exit loop 
+                   cat('h has NaN, in typeB fk_old used to compute h', fk_old)
+	    		stopit <- TRUE
                     break
                 }
-                                        #if(i==70 && k==iter) browser()
-		RET <- glmreg_fit(x=x.act*sqrt(B), y=h*sqrt(B), weights=weights, offset=offset, lambda=lambda[i],alpha=alpha,gamma=gamma, rescale=FALSE, standardize=FALSE, penalty.factor = penalty.factor.act, maxit=maxit, eps=eps, family="gaussian", penalty=penalty, start=start.act)
-		RET$b0 <- RET$b0/sqrt(B)
-### for LASSO, the above two lines are equivalent to the next line
+               RET <- .Fortran("glmreg_fit_fortran",
+	                        x=as.double(x.act), 
+				y=as.double(h), 
+				weights=as.double(weights),
+			        n=as.integer(n),
+			        m=as.integer(m.act),	
+				start=as.double(start.act), 
+				etastart=as.double(etastart),
+	                        mustart=as.double(mustart),
+			       	offset=as.double(offset),
+			       	nlambda=as.integer(1),
+			       	lambda=as.double(lambda[i]/B), 
+				alpha=as.double(alpha),        
+			        gam=as.double(gamma), 
+				rescale=as.integer(0),
+			       	standardize=as.integer(0),
+				penaltyfactor=as.double(penalty.factor.act),
+				thresh=as.double(thresh), 
+				epsbino=as.double(0), 
+				maxit=as.integer(maxit),
+			        eps=as.double(eps), 
+				theta=as.double(0),
+			       	family=as.integer(1), 
+				penalty=as.integer(pentype), 
+				trace=as.integer(trace),
+				beta=as.double(matrix(0, ncol=1, nrow=m)),
+				b0=as.double(0),
+				yhat=as.double(rep(0, n)),
+				PACKAGE="mpath")
+     # epsbino, theta are not used in the above Fortran call with family="gaussian"
+#		RET <- glmreg_fit(x=x.act*sqrt(B), y=h*sqrt(B), weights=weights, offset=offset, lambda=lambda[i],alpha=alpha,gamma=gamma, rescale=FALSE, standardize=FALSE, penalty.factor = penalty.factor.act, maxit=maxit, eps=eps, family="gaussian", penalty=penalty, start=start.act)
+#		RET$b0 <- RET$b0/sqrt(B)  ### this line should be removed after calling glmreg_fit_fortran
+### for LASSO, the above two lines are equivalent to the next line if and only if standardize=FALSE
                                         #RET <- glmreg_fit(x=x, y=h, weights=weights, lambda=lambda[i]/B,alpha=alpha,gamma=gamma, rescale=FALSE, standardize=FALSE, penalty.factor = penalty.factor, maxit=maxit, eps=eps, family="gaussian", penalty=penalty)
-                fk <- RET$fitted.values <- predict(RET, newx=x.act)
-                start.act <- coef(RET)
-                los[k, i] <- mean(loss(y, f=fk, cost, family = rfamily, s=s, fk=NULL))
+#                fk <- RET$fitted.values <- predETt(RET, newx=x.act)
+#                start.act <- coef(RET)
+                fk <- RET$fitted.values <- RET$yhat
+                start.act <- c(RET$b0, RET$beta)
+                etastart <- RET$yhat
+		mustart <- RET$mustart
+	  	los[k, i] <- mean(loss(y, f=fk, cost, family = rfamily, s=s, fk=NULL))
 ###penalized loss value for beta
+if(any(is.na(RET$beta))){
+                cat('rfamily=', rfamily, 's=', s, 'B=', B, "\n")
+		cat('fk_old', fk_old, "\n")
+	        cat("h=", h, "\n")
+	        cat("start.act=", start.act, "\n")
+	        cat("mustart=", mustart, "\n")
+	        cat("etastart=", etastart, "\n")
+}
                 penval <- .Fortran("penGLM",
                                    start=as.double(RET$beta),
                                    m=as.integer(m.act),
-                                   lambda=as.double(lambda[i]*penalty.factor.act),
+                                   lambda=as.double(lambda[i]/B*penalty.factor.act),
                                    alpha=as.double(alpha),
                                    gam=as.double(gamma),
                                    penalty=as.integer(pentype),
                                    pen=as.double(0.0),
                                    PACKAGE="mpath")$pen
-                if(standardize)  ### lambda value, hence penval, depends on whether standardize is TRUE/FALSE
+     		if(standardize)  ### lambda value, hence penval, depends on whether standardize is TRUE/FALSE
                     pll[k, i] <- los[k, i] + n*penval
                 else pll[k, i] <- los[k, i] + penval
                 d1 <- sum((fk_old - fk)^2)
@@ -372,7 +435,56 @@ nclreg_fit <- function(x,y, weights, offset=NULL, cost=0.5, rfamily=c("clossR",
         }
         list(beta=beta, b0=b0, RET=RET, risk=los, pll=pll)
     }
-### update for one element of lambda depending on direction="fwd" (last element of lambda) or "bwd" (then first element of lambda) in each MM iteration, and iterate until convergency of prediction. Then fit a solution path based on the sequence of lambda.
+#typeBB was converted from typeB, works for both "bwd" and "fwd".
+### these number conversion should be consistent to those in Fortran subroutines src/compute_h and loss
+    rfamilytype <- switch(rfamily,       
+                    "clossR"=11,
+                    "closs"=12,
+                    "gloss"=13,
+                    "qloss"=14
+                    )
+    typeBB <- function(beta, b0){
+	    RET <- .Fortran("nclreg_fortran",
+			    x=as.double(x), 
+			    y=as.double(y),
+			    weights=as.double(weights),
+			    n=as.integer(n),
+			    m=as.integer(m),
+			    start=as.double(start), 
+			    etastart=as.double(etastart),
+			    mustart=as.double(mustart),
+			    offset=as.double(offset),
+			    nlambda=as.integer(nlambda), 
+			    lambda=as.double(lambda), 
+			    alpha=as.double(alpha),
+		            gam=as.double(gamma), 
+			    standardize=as.integer(0), 
+			    penaltyfactor=as.double(penalty.factor),
+			    maxit=as.integer(maxit), 
+			    eps=as.double(eps), 
+			    family=as.integer(1), 
+			    penalty=as.integer(pentype), 
+			    trace=as.integer(trace), 
+			    beta=as.double(beta),
+		            b0=as.double(b0), 
+			    yhat=as.double(RET$fitted.values), 
+			    iter=as.integer(iter),
+			    del=as.double(del), 
+			    rfamily=as.integer(rfamilytype),
+			    B=as.double(B), 
+			    s=as.double(s),
+			    los=as.double(rep(0, nlambda)),
+			    pll=as.double(rep(0, nlambda)),
+			    rescale=as.integer(0),
+			    thresh=as.double(thresh),
+			    epsbino=as.double(0),
+			    theta=as.double(0),
+			    cost=as.double(cost),
+			    active=as.integer(active),
+			    PACKAGE="mpath")
+       	    list(beta=matrix(RET$beta, ncol=nlambda), b0=RET$b0, RET=RET, risk=RET$los, pll=RET$pll)
+    }
+### update for one element of lambda depending on increasing sequence of lambda (last element of lambda) or decreasing sequence of lambda (then first element of lambda) in each MM iteration, and iterate until convergency of prediction. Then fit a solution path based on the sequence of lambda. Experiment code. Argument direction has been removed.
     typeC <- function(beta, b0){
         if(trace) {
             cat("Quadratic majorization iterations ...\n")
@@ -381,7 +493,7 @@ nclreg_fit <- function(x,y, weights, offset=NULL, cost=0.5, rfamily=c("clossR",
         k <- 1
         fk <- RET$fitted.values
         d1 <- 10
-        lam <- lambda[ifelse(direction=="fwd", nlambda, 1)]
+        lam <- lambda[ifelse(decreasing, nlambda, 1)]
 	while(d1 > del && k <= iter){
 	    fk_old <- fk
             h <- compute.h(rfamily, y, fk_old, s, B)
@@ -417,8 +529,11 @@ nclreg_fit <- function(x,y, weights, offset=NULL, cost=0.5, rfamily=c("clossR",
         b0 <- RET$b0
         list(beta=beta, b0=b0, RET=RET, risk=los, pll=pll)
     }
-    if(type.path=="active") tmp <- typeB(beta, b0)
-    else if(type.path=="naive") tmp <- typeA(beta, b0)
+#    if(type.path=="active") tmp <- typeB(beta, b0)
+# typeA and typeB are combined into typeBB
+    if(type.path=="active" || type.path=="nonactive") tmp <- typeBB(beta, b0)
+#    if(type.path=="active") tmp <- typeBB(beta, b0)
+#    else if(type.path=="nonactive") tmp <- typeA(beta, b0)
     else if(type.path=="onestep") tmp <- typeC(beta, b0)
     beta <- tmp$beta
     b0 <- tmp$b0
@@ -447,7 +562,7 @@ nclreg_fit <- function(x,y, weights, offset=NULL, cost=0.5, rfamily=c("clossR",
     RET$type.init <- type.init
     RET$mstop.init <- mstop.init
     RET$nu.init <- nu.init
-    RET$direction <- direction
+    RET$decreasing <- decreasing
     RET$type.path <- type.path
     class(RET) <- "nclreg"
     RET
